import router from './router'
import store from './store'
import { Message } from 'element-ui'
import NProgress from 'nprogress' // 顶部加载条
import 'nprogress/nprogress.css' // 顶部加载条css
import { getToken } from '@/utils/auth' // 获取token
import getPageTitle from '@/utils/get-page-title' // 获取title
import Layout from '@/layout'
// 获取组件的方法
const _import = require('./router/_import_' + process.env.NODE_ENV)

NProgress.configure({ showSpinner: false }) // 加载条

const whiteList = ['/login'] // 白名单

// 路由拦截-前
router.beforeEach(async(to, from, next) => {
  NProgress.start() // 开始加载
  document.title = getPageTitle(to.meta.title) // 页面title
  // 优先检测token
  const hasToken = getToken()

  if (!hasToken) { // token不存在
    if (to.path && whiteList.indexOf(to.path) !== -1) { // 在白名单内,放行
      next()
    } else {
      next(`/login`)
      NProgress.done() // 结束加载
    }
  } else { // token存在
    const { matched } = to // 检查是否匹配到了路由(为了解决退出再次登录不加载)
    if (matched.length === 0) { // 没有匹配到路由
      try {
        await store.dispatch('user/getInfo').then(() => { // 请求用户信息
          if (store.getters.routes.length < 1) {
            global.antRouter = []
            next()
          } else {
            const menus = filterAsyncRouter(JSON.parse(JSON.stringify(store.getters.routes))) // 1.过滤路由
            router.addRoutes(menus) // 2.动态添加路由
            global.antRouter = menus // 3.将路由数据传递给全局变量，做侧边栏菜单渲染工作
            next({
              ...to,
              replace: true
            })
          }
        })
      } catch (e) {
        NProgress.done() // 结束加载
        Message.error(e)
      }
    } else { // 匹配到路由,继续执行
      const hasGetUserInfo = store.getters.name // 查看是否存在用户信息
      if (hasGetUserInfo) { // 存在,继续
        next()
      } else { // 不存在,请求获取用户信息
        try {
          await store.dispatch('user/getInfo').then(() => { // 请求用户信息
            if (store.getters.routes.length < 1) {
              global.antRouter = []
              next()
            } else {
              const menus = filterAsyncRouter(JSON.parse(JSON.stringify(store.getters.routes))) // 1.过滤路由
              router.addRoutes(menus) // 2.动态添加路由
              global.antRouter = menus // 3.将路由数据传递给全局变量，做侧边栏菜单渲染工作
              next({
                ...to,
                replace: true
              })
            }
          })
        } catch (e) {
          NProgress.done() // 结束加载
          Message.error(e)
        }
      }
    }
  }
})

// 路由拦截-后
router.afterEach(() => {
  NProgress.done()
})

// 动态加载路由
function filterAsyncRouter(asyncRouterMap) {
  const accessedRouters = asyncRouterMap.filter(route => {
    if (route.page) {
      if (route.page === 'Layout') {
        route.component = Layout
      } else {
        route.component = _import(route.page)
      }
    }
    if (route.children && route.children.length) {
      route.children = filterAsyncRouter(route.children)
    }
    return true
  })
  return accessedRouters
}
