import { login, logout, getInfo } from '@/api/user'
import { getToken, setToken, removeToken } from '@/utils/auth'
import { resetRouter } from '@/router'
import asyncRouterMap from '@/router/asyncrouter'
import Cookies from 'js-cookie'

const state = {
  token: getToken(),
  name: '',
  avatar: 'https://wpimg.wallstcn.com/f778738c-e4f8-4870-b634-56703b4acafe.gif',
  routes: [],
  roles: '',
  teamId: ''
}

const mutations = {
  SET_TOKEN: (state, token) => {
    state.token = token
  },
  SET_NAME: (state, name) => {
    state.name = name
  },
  SET_AVATAR: (state, avatar) => {
    state.avatar = avatar
  },
  SET_ROUTES: (state, routes) => {
    state.routes = routes
  },
  SET_ROLES: (state, roles) => {
    state.roles = roles
  },
  SET_TEAMID: (state, teamId) => {
    state.teamId = teamId
  }
}

const actions = {
  // 登录
  login({ commit }, userInfo) {
    const { name, password } = userInfo
    return new Promise((resolve, reject) => {
      login({ name: name.trim(), password: password }).then(response => {
        const { data } = response
        commit('SET_TOKEN', data) // 提交vuex存储token
        setToken(data) // cookie 存储token
        resolve()
      }).catch(error => {
        reject(error)
      })
    })
  },

  // 获取用户信息（权限）
  getInfo({ commit, state }) {
    return new Promise((resolve, reject) => {
      getInfo(state.token).then(response => {
        const { data } = response

        if (!data) {
          reject('验证失败,请推出重新登陆')
        }
        const { name, is_team, team_id } = data
        let { permissions } = data

        commit('SET_NAME', name)

        // 验证是否是管理员 permissions=[0]
        if (typeof permissions === 'string') {
          try {
            permissions = JSON.parse(permissions)
          } catch (e) {
            permissions = []
          }
        }

        // 如果是管理员 获得所有权限
        if (permissions.length > 0 && permissions[0] === 0) {
          permissions = asyncRouterMap
        }

        // 提交路由到vuex store
        commit('SET_ROUTES', permissions)
        commit('SET_ROLES', is_team)
        commit('SET_TEAMID', team_id)
        resolve(data)
      }).catch(error => {
        reject(error)
      })
    })
  },

  // 退出登录
  logout({ commit, state }) {
    return new Promise((resolve, reject) => {
      logout().then(response => {
        const { error_code } = response
        if (error_code === 200) {
          commit('SET_TOKEN', '') // 清除token
          commit('SET_NAME', '') // 清除name
          commit('SET_ROUTES', []) // 清除routes
          removeToken()
          resetRouter()
          Cookies.remove('sidebarStatus')
        }
        resolve(response)
      }).catch(error => {
        reject(error)
      })
    })
  },

  // 重置 token
  resetToken({ commit }, token) {
    console.log(token)
    if (!token) {
      token = ''
    }
    commit('SET_TOKEN', token)
    setToken(token)
  },

  // remove token
  removeToken({ commit }) {
    commit('SET_TOKEN', '')
    removeToken()
  }
}

export default {
  namespaced: true,
  state,
  mutations,
  actions
}

