import axios from 'axios'
import { MessageBox, Message } from 'element-ui'
import store from '@/store'
import { getToken, setToken, removeToken } from '@/utils/auth'

axios.defaults.headers.post['Content-Type'] = 'application/x-www-form-urlencoded'

// 创建axios
const service = axios.create({
  baseURL: process.env.VUE_APP_BASE_API,
  withCredentials: false,
  timeout: 500000 // 超时
})

// 请求拦截
service.interceptors.request.use(
  config => {
    if (getToken()) {
      // 获取cookie存储的token进行授权
      config.headers['authorization'] = 'Bearer ' + getToken()
    }
    return config
  },
  error => {
    console.log(error) // for debug
    return Promise.reject(error)
  }
)

// 响应拦截
service.interceptors.response.use(
  response => {
    const { headers } = response
    if (headers.authorization) { // 如果存在header.authorization则刷新token
      const r_token = headers.authorization.substring(7)
      setToken(r_token)
      console.log(r_token)
      store.dispatch('user/resetToken', r_token)
    }

    const res = response.data

    const { error_code } = res
    // 监听错误码err_code
    // 200-请求正常
    if (error_code !== 200) {
      if (error_code === 401) {
        store.dispatch('user/removeToken')
        MessageBox.confirm('请重新登录', '登录信息过期', {
          confirmButtonText: '重新登录',
          cancelButtonText: '取消',
          type: 'warning'
        }).then(() => {
          store.dispatch('user/resetToken').then(() => {
            location.reload()
          })
        }, () => {
          store.dispatch('user/resetToken')
          Message({
            message: '登录信息过期，请重新登录（刷新页面或点击右上角头像退出登陆即可）',
            duration: 10000 * 1000,
            type: 'error'
          })
        })
      } else {
        Message({
          message: res.msg || 'Error',
          type: 'error',
          duration: 2 * 1000
        })
      }
      return res
    } else {
      // token刷新
      const { headers } = response
      if (headers.authorization) {
        const r_token = headers.authorization.substring(7)
        store.dispatch('user/resetToken', r_token)
      }
      return res
    }
  },
  error => {
    if (error && error.response) {
      switch (error.response.status) {
        case 401 :
          error.message = '您还未登录'
          Message({
            message: error.message + '即将跳转登陆',
            type: 'error',
            duration: 3 * 1000
          })
          setTimeout(() => {
            removeToken()
            store.dispatch('user/removeToken') // 清除token
            window.location.reload()
          }, 3000)
          return
        case 500:
          error.message = 'token过期,请重新登陆'
          return
        default:
          error.message = `连接错误${error.response.status}`
      }
    } else {
      error.message = '网络出现问题,请稍后再试'
    }
    Message({
      message: error.message,
      type: 'error',
      duration: 5 * 1000
    })
    return Promise.reject(error)
  }
)

export default service
